/**
 * Feathery FTP-Server <https://sourceforge.net/projects/feathery>
 * Copyright (C) 2005-2010 Andreas Martin (andreas.martin@linuxmail.org)
 *
 * ftpMessages.h - FTP-Server messages
 *
 * This header exports the in ftpMessges.c defined messages.
 *
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef FTPMESSAGES_H_
#define FTPMESSAGES_H_

extern const char ftpMsg000[];
extern const char ftpMsg001[];
extern const char ftpMsg002[];
extern const char ftpMsg003[];
extern const char ftpMsg004[];
extern const char ftpMsg005[];
extern const char ftpMsg006[];
extern const char ftpMsg007[];
extern const char ftpMsg008[];
extern const char ftpMsg009[];
extern const char ftpMsg010[];
extern const char ftpMsg011[];
extern const char ftpMsg012[];
extern const char ftpMsg013[];
extern const char ftpMsg014[];
extern const char ftpMsg015[];
extern const char ftpMsg016[];
extern const char ftpMsg018[];
extern const char ftpMsg019[];
extern const char ftpMsg020[];
extern const char ftpMsg021[];
extern const char ftpMsg022[];
extern const char ftpMsg023[];
extern const char ftpMsg024[];
extern const char ftpMsg025[];
extern const char ftpMsg026[];
extern const char ftpMsg027[];
extern const char ftpMsg028[];
extern const char ftpMsg029[];
extern const char ftpMsg030[];
extern const char ftpMsg031[];
extern const char ftpMsg032[];
extern const char ftpMsg033[];
extern const char ftpMsg034[];
extern const char ftpMsg035[];
extern const char ftpMsg036[];
extern const char ftpMsg037[];
extern const char ftpMsg038[];
extern const char ftpMsg039[];
extern const char ftpMsg040[];


#endif /* FTPMESSAGES_H_ */
