// ==============================================================
//	This file is part of Glest Shared Library (www.glest.org)
//
//	Copyright (C) 2009-2010 Titus Tscharntke (info@titusgames.de) and
//                          Mark Vejvoda (mark_vejvoda@hotmail.com)
//
//	You can redistribute this code and/or modify it under
//	the terms of the GNU General Public License as published
//	by the Free Software Foundation; either version 2 of the
//	License, or (at your option) any later version
// ==============================================================
#ifndef _SHARED_PLATFORMCOMMON_MATHWRAPPER_H_
#define _SHARED_PLATFORMCOMMON_MATHWRAPPER_H_

#ifdef USE_STREFLOP

#include <cmath>
#include <streflop_cond.h>

#else

#include <cmath>

#endif

#include "leak_dumper.h"

#endif
