// ==============================================================
//	This file is part of Glest (www.glest.org)
//
//	Copyright (C) 2010 Frank Tetzel
//
//	You can redistribute this code and/or modify it under
//	the terms of the GNU General Public License as published
//	by the Free Software Foundation; either version 2 of the
//	License, or (at your option) any later version
// ==============================================================

#ifndef _MAPEDITOR_ICONS_H_
#define _MAPEDITOR_ICONS_H_

#include "icons/radius_1.xpm"
#include "icons/radius_2.xpm"
#include "icons/radius_3.xpm"
#include "icons/radius_4.xpm"
#include "icons/radius_5.xpm"
#include "icons/radius_6.xpm"
#include "icons/radius_7.xpm"
#include "icons/radius_8.xpm"
#include "icons/radius_9.xpm"

#include "icons/brush_surface_custom.xpm"
#include "icons/brush_surface_grass1.xpm"
#include "icons/brush_surface_grass2.xpm"
#include "icons/brush_surface_road.xpm"
#include "icons/brush_surface_stone.xpm"

#include "icons/brush_none.xpm"
#include "icons/brush_object_bush.xpm"
#include "icons/brush_object_dead-tree.xpm"
#include "icons/brush_object_stone.xpm"
#include "icons/brush_object_tree.xpm"
#include "icons/brush_object_water-object.xpm"
#include "icons/brush_object_c1-bigtree.xpm"
#include "icons/brush_object_c2-hanged.xpm"
#include "icons/brush_object_c4-bigrock.xpm"
#include "icons/brush_object_c5-blocking.xpm"
#include "icons/brush_object_c3-statue.xpm"

#include "icons/brush_gradient_0.xpm"
#include "icons/brush_gradient_n1.xpm"
#include "icons/brush_gradient_n2.xpm"
#include "icons/brush_gradient_n3.xpm"
#include "icons/brush_gradient_n4.xpm"
#include "icons/brush_gradient_n5.xpm"
#include "icons/brush_gradient_p1.xpm"
#include "icons/brush_gradient_p2.xpm"
#include "icons/brush_gradient_p3.xpm"
#include "icons/brush_gradient_p4.xpm"
#include "icons/brush_gradient_p5.xpm"
#include "icons/brush_height_0.xpm"
#include "icons/brush_height_n1.xpm"
#include "icons/brush_height_n2.xpm"
#include "icons/brush_height_n3.xpm"
#include "icons/brush_height_n4.xpm"
#include "icons/brush_height_n5.xpm"
#include "icons/brush_height_p1.xpm"
#include "icons/brush_height_p2.xpm"
#include "icons/brush_height_p3.xpm"
#include "icons/brush_height_p4.xpm"
#include "icons/brush_height_p5.xpm"

#include "icons/brush_players_blue.xpm"
#include "icons/brush_players_player.xpm"
#include "icons/brush_players_green.xpm"
#include "icons/brush_players_yellow.xpm"
#include "icons/brush_players_red.xpm"
#include "icons/brush_players_white.xpm"
#include "icons/brush_players_cyan.xpm"
#include "icons/brush_players_orange.xpm"
#include "icons/brush_players_pink.xpm"

#include "icons/brush_resource_1-gold.xpm"
#include "icons/brush_resource_2-stone.xpm"
#include "icons/brush_resource_3.xpm"
#include "icons/brush_resource_4.xpm"
#include "icons/brush_resource_5.xpm"

#include "icons/edit_randomize-heights.xpm"
// #include "icons/edit_randomize.xpm"
#include "icons/edit_redo.xpm"
#include "icons/edit_switch-surfaces.xpm"
#include "icons/edit_undo.xpm"

#endif
