// ==============================================================
//	This file is part of Glest Shared Library (www.glest.org)
//
//	Copyright (C) 2001-2005 Martio Figueroa
//
//	You can redistribute this code and/or modify it under 
//	the terms of the GNU General Public License as published 
//	by the Free Software Foundation; either version 2 of the 
//	License, or (at your option) any later version
// ==============================================================

#ifndef _SHARED_PLATFORM_DEFINITIONS_H_
#define _SHARED_PLATFORM_DEFINITIONS_H_

#include "leak_dumper.h"

namespace Shared{ namespace Platform{



}}//end namespace

#endif
