-- Lines starting with "--" are comments in this file.
-- You should add your installer's strings here, instead of localization.lua,
--  but localization.lua has instructions on how this file is formatted.

MojoSetup.applocalization = {
    ["Uninstall complete"] = {
     	en = "Uninstall complete. You may also delete '.megaglest' in your home directory to remove personal settings.",
        de = "Deinstallation abgeschlossen. Um auch Deine MegaGlest-Konfiguration zu entfernen kannst Du jetzt noch das Verzeichnis '.megaglest' in Deinem Home-Verzeichnis löschen.",
        es = "Desinstalación completa.",
        pl = "Usuwanie zakończone. Możesz usunąć '.megaglest' w katalogu domowym by usunąć zapisane gry i ustawienia.",
        it = "Disinstallazione completata. Se desideri cancellare i salvataggi e le impostazioni cancella '.megaglest' nella tua directory home.",
    };

    ["MegaGlest README"] = {
        en = "MegaGlest README",
        de = "MegaGlest README-Datei",
        es = "LEEME",
        pl = "MegaGlest README",
         it = "Informazioni su MegaGlest",
    };

    ["docs/README"] = {
    };

    ["MegaGlest Game License"] = {
        en = "MegaGlest Game Licence",
        de = "MegaGlest-Lizenz (Spiel)",
        es = "Licencia de uso de MegaGlest",
        pl = "Licencja gry MegaGlest",
        it = "Licenza di MegaGlest",
    };

    ["MegaGlest Data License"] = {
        en = "MegaGlest Data Licence",
        de = "MegaGlest-Lizenz (Daten)",
        es = "Licencia de uso de MegaGlest",
        pl = "Licencja danych MegaGlest",
        it = "Licenza di MegaGlest",
    };

    ["MegaGlest Uninstall Title"] = {
        en = "Remove previous version",
        de = "Entfernen der vorherigen Version",
        es = "Remove previous version",
        pl = "Usuń poprzednią wersję",
        it = "Rimuovere la versione precedente",
    };

    ["MegaGlest Uninstall Prompt"] = {
        en = "We have detected a previous installation of MegaGlest, should we remove it?",
        de = "Es wurde eine frühere Installation von MegaGlest erkannt. Möchtest Du sie entfernen?",
        es = "We have detected a previous installation of MegaGlest, should we remove it?",
        pl = "Wykryto poprzednią instalację MegaGlest, usunąć ją?",
        it = "E' stata trovata una versione meno recente di MegaGlest, vuoi rimuoverla?",
    };

    ["MegaGlest Visit Website Title"] = {
        en = "Visit megaglest.org",
        de = "Besuche megaglest.org",
        es = "Visit megaglest.org",
        pl = "Odwiedź megaglest.org",
        it = "Visita megaglest.org",
    };

    ["MegaGlest Visit Website Prompt"] = {
        en = "Would you like to view our getting started page on megaglest.org?",
        de = "Möchtest Du unseren 'Getting Started'-Guide auf megaglest.org ansehen?",
        es = "Would you like to view our getting started page on megaglest.org?",
        pl = "Czy chcesz zobaczyć stronę z pierwszymi krokami na megaglest.org?",
        it = "Vorresti visualizzare la nostra pagina di benvenuto su megaglest.org?",
    };

    ["docs/LICENSE"] = {
    };

};

-- end of app_localization.lua ...

