/* This file is stub automatically generated by import.pl
 *
    Copyright 2006 Nicolas Brodu
    		  2010 Mark Vejvoda
    Code released according to the GNU Lesser General Public License
*/

// Include bridge, just in case math_private is not included
#include "../streflop_libm_bridge.h"

